@@ -1,5 +1,5 @@
 /*
- * "$Id: usb-libusb.c 9582 2011-03-04 19:28:38Z mike $"
+ * "$Id: usb-libusb.c 9831 2011-06-14 23:03:29Z mike $"
  *
  *   Libusb interface code for CUPS.
  *
@@ -31,6 +31,7 @@
 
 #include <usb.h>
 #include <poll.h>
+#include <cups/cups-private.h>
 
 
 /*
@@ -113,8 +114,8 @@ print_device(const char *uri,		/* I - Device URI */
 
   while ((printer = find_device(print_cb, uri)) == NULL)
   {
-    _cupsLangPuts(stderr,
-		  _("INFO: Waiting for printer to become available...\n"));
+    _cupsLangPrintFilter(stderr, "INFO",
+			 _("Waiting for printer to become available."));
     sleep(5);
   }
 
@@ -174,9 +175,8 @@ print_device(const char *uri,		/* I - Device URI */
 	  if (usb_bulk_write(printer->handle, printer->write_endp, buffer,
 	                        bytes, 3600000) < 0)
 	  {
-	    _cupsLangPrintf(stderr,
-			    _("ERROR: Unable to write %d bytes to printer!\n"),
-			    (int)bytes);
+	    _cupsLangPrintFilter(stderr, "ERROR",
+			         _("Unable to send data to printer."));
 	    tbytes = -1;
 	    break;
 	  }
@@ -505,6 +505,7 @@ make_device_uri(
 		*mdl,			/* Model */
 		*des,			/* Description */
 		*sern;			/* Serial number */
+  size_t	mfglen;			/* Length of manufacturer string */
   char		tempmfg[256],		/* Temporary manufacturer string */
 		tempsern[256],		/* Temporary serial number string */
 		*tempptr;		/* Pointer into temp string */
@@ -514,7 +515,7 @@ make_device_uri(
   * Get the make, model, and serial numbers...
   */
 
-  num_values = _ppdGet1284Values(device_id, &values);
+  num_values = _cupsGet1284Values(device_id, &values);
 
   if ((sern = cupsGetOption("SERIALNUMBER", num_values, values)) == NULL)
     if ((sern = cupsGetOption("SERN", num_values, values)) == NULL)
@@ -558,9 +559,9 @@ make_device_uri(
 
   if (mfg)
   {
-    if (!strcasecmp(mfg, "Hewlett-Packard"))
+    if (!_cups_strcasecmp(mfg, "Hewlett-Packard"))
       mfg = "HP";
-    else if (!strcasecmp(mfg, "Lexmark International"))
+    else if (!_cups_strcasecmp(mfg, "Lexmark International"))
       mfg = "Lexmark";
   }
   else
@@ -584,6 +585,16 @@ make_device_uri(
     mfg = tempmfg;
   }
 
+  mfglen = strlen(mfg);
+
+  if (!strncasecmp(mdl, mfg, mfglen) && _cups_isspace(mdl[mfglen]))
+  {
+    mdl += mfglen + 1;
+
+    while (_cups_isspace(*mdl))
+      mdl ++;
+  }
+
  /*
   * Generate the device URI from the manufacturer, model, serial number,
   * and interface number...
@@ -736,7 +747,87 @@ print_cb(usb_printer_t *printer,	/* I - Printer */
          const char    *device_id,	/* I - IEEE-1284 device ID */
          const void    *data)		/* I - User data (make, model, S/N) */
 {
-  return (!strcmp((char *)data, device_uri));
+  char	requested_uri[1024],		/* Requested URI */
+	*requested_ptr,			/* Pointer into requested URI */
+	detected_uri[1024],		/* Detected URI */
+	*detected_ptr;			/* Pointer into detected URI */
+
+
+ /*
+  * If we have an exact match, stop now...
+  */
+
+  if (!strcmp((char *)data, device_uri))
+    return (1);
+
+ /*
+  * Work on copies of the URIs...
+  */
+
+  strlcpy(requested_uri, (char *)data, sizeof(requested_uri));
+  strlcpy(detected_uri, device_uri, sizeof(detected_uri));
+
+ /*
+  * libusb-discovered URIs can have an "interface" specification and this
+  * never happens for usblp-discovered URIs, so remove the "interface"
+  * specification from the URI which we are checking currently. This way a
+  * queue for a usblp-discovered printer can now be accessed via libusb.
+  *
+  * Similarly, strip "?serial=NNN...NNN" as needed.
+  */
+
+  if ((requested_ptr = strstr(requested_uri, "?interface=")) == NULL)
+    requested_ptr = strstr(requested_uri, "&interface=");
+  if ((detected_ptr = strstr(detected_uri, "?interface=")) == NULL)
+    detected_ptr = strstr(detected_uri, "&interface=");
+
+  if (!requested_ptr && detected_ptr)
+  {
+   /*
+    * Strip "[?&]interface=nnn" from the detected printer.
+    */
+
+    *detected_ptr = '\0';
+  }
+  else if (requested_ptr && !detected_ptr)
+  {
+   /*
+    * Strip "[?&]interface=nnn" from the requested printer.
+    */
+
+    *requested_ptr = '\0';
+  }
+
+  if ((requested_ptr = strstr(requested_uri, "?serial=?")) != NULL)
+  {
+   /*
+    * Strip "?serial=?" from the requested printer.  This is a special
+    * case, as "?serial=?" means no serial number and not the serial
+    * number '?'.  This is not covered by the checks below...
+    */
+
+    *requested_ptr = '\0';
+  }
+
+  if ((requested_ptr = strstr(requested_uri, "?serial=")) == NULL &&
+      (detected_ptr = strstr(detected_uri, "?serial=")) != NULL)
+  {
+   /*
+    * Strip "?serial=nnn" from the detected printer.
+    */
+
+    *detected_ptr = '\0';
+  }
+  else if (requested_ptr && !detected_ptr)
+  {
+   /*
+    * Strip "?serial=nnn" from the requested printer.
+    */
+
+    *requested_ptr = '\0';
+  }
+
+  return (!strcmp(requested_uri, detected_uri));
 }
 
 
@@ -777,9 +868,8 @@ side_cb(usb_printer_t *printer,		/* I - Printer */
 	    while (usb_bulk_write(printer->handle, printer->write_endp, buffer,
 				  bytes, 5000) < 0)
 	    {
-	      _cupsLangPrintf(stderr,
-			      _("ERROR: Unable to write %d bytes to printer!\n"),
-			      (int)bytes);
+	      _cupsLangPrintFilter(stderr, "ERROR",
+			           _("Unable to send data to printer."));
 	      tbytes = -1;
 	      break;
 	    }
@@ -830,6 +920,6 @@ side_cb(usb_printer_t *printer,		/* I - Printer */
 
 
 /*
- * End of "$Id: usb-libusb.c 9582 2011-03-04 19:28:38Z mike $".
+ * End of "$Id: usb-libusb.c 9831 2011-06-14 23:03:29Z mike $".
  */
 
