@@ -1,5 +1,5 @@
 /*
- * "$Id: testraster.c 9598 2011-03-11 23:53:27Z mike $"
+ * "$Id: testraster.c 9597 2011-03-11 23:53:20Z mike $"
  *
  *   Raster test program routines for CUPS.
  *
@@ -223,6 +223,7 @@ main(int  argc,				/* I - Number of command-line args */
     errors = do_ps_tests();
     errors += do_raster_tests(CUPS_RASTER_WRITE);
     errors += do_raster_tests(CUPS_RASTER_WRITE_COMPRESSED);
+    errors += do_raster_tests(CUPS_RASTER_WRITE_PWG);
   }
   else
   {
@@ -539,7 +540,8 @@ do_raster_tests(cups_mode_t mode)	/* O - Write mode */
 
   printf("cupsRasterOpen(%s): ",
          mode == CUPS_RASTER_WRITE ? "CUPS_RASTER_WRITE" :
-	     "CUPS_RASTER_WRITE_COMPRESSED");
+	     mode == CUPS_RASTER_WRITE ? "CUPS_RASTER_WRITE_COMPRESSED" :
+	                                 "CUPS_RASTER_WRITE_PWG");
   fflush(stdout);
 
   if ((fp = fopen("test.raster", "wb")) == NULL)
@@ -1072,5 +1074,5 @@ print_changes(
 
 
 /*
- * End of "$Id: testraster.c 9598 2011-03-11 23:53:27Z mike $".
+ * End of "$Id: testraster.c 9597 2011-03-11 23:53:20Z mike $".
  */
